package edu.msoe.se1011.houseestimator;

import android.support.annotation.Nullable;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import FIXME.BuildingCostEstimator;

public class MainActivity extends AppCompatActivity {

    private EditText squareFeet;
    private EditText fullBaths;
    private EditText halfBaths;
    private EditText bedrooms;
    private EditText windows;
    private EditText garageSpaces;
    private Button calculate;
    private TextView output;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Connect UI elements
        squareFeet = (EditText)findViewById(R.id.squareFeet);
        fullBaths = (EditText)findViewById(R.id.fullBaths);
        halfBaths = (EditText)findViewById(R.id.halfBaths);
        bedrooms = (EditText)findViewById(R.id.bedrooms);
        windows = (EditText)findViewById(R.id.windows);
        garageSpaces = (EditText)findViewById(R.id.garageSpaces);
        calculate = (Button)findViewById(R.id.calculate);
        output = (TextView)findViewById(R.id.output);

        // Ensure calculate button is disabled if any field is blank
        squareFeet.addTextChangedListener(textWatcher);
        fullBaths.addTextChangedListener(textWatcher);
        halfBaths.addTextChangedListener(textWatcher);
        bedrooms.addTextChangedListener(textWatcher);
        windows.addTextChangedListener(textWatcher);
        garageSpaces.addTextChangedListener(textWatcher);

        // Make button respond to click
        calculate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                calculateCost();
            }
        });
    }

    private void calculateCost() {
        int numSquareFeet = extractInt(squareFeet);
        int numFullBaths = extractInt(fullBaths);
        int numHalfBaths = extractInt(halfBaths);
        int numBedrooms = extractInt(bedrooms);
        int numWindows = extractInt(windows);
        double numGarageSpaces = extractDouble(garageSpaces);

        String displayText = ensureValidEntries(numSquareFeet, numFullBaths, numHalfBaths, numBedrooms, numWindows, numGarageSpaces);
        if(displayText==null) {
            BuildingCostEstimator estimator = new BuildingCostEstimator();
            estimator.setSquareFeet(numSquareFeet);
            estimator.setNumFullBaths(numFullBaths);
            estimator.setNumHalfBaths(numHalfBaths);
            estimator.setNumBedrooms(numBedrooms);
            estimator.setNumWindows(numWindows);
            estimator.setNumGarages(numGarageSpaces);
            if(estimator.getNumGarages()<2.0) {
                displayText = getString(R.string.cold_weather_warning);
            } else {
                displayText = getString(R.string.total_cost) + estimator.costToBuild();
            }
        }
        output.setText(displayText);
        output.setVisibility(View.VISIBLE);
    }

    @Nullable
    private String ensureValidEntries(int numSquareFeet, int numFullBaths, int numHalfBaths, int numBedrooms, int numWindows, double numGarageSpaces) {
        String displayText = null;
        if(numSquareFeet<0) {
            displayText = "Square Feet";
        }
        if(numFullBaths<0) {
            displayText = "Full Bathrooms";
        }
        if(numHalfBaths<0) {
            displayText = "Half Bathrooms";
        }
        if(numBedrooms<0) {
            displayText = "Bedrooms";
        }
        if(numWindows<0) {
            displayText = "Windows";
        }
        if(numGarageSpaces<0) {
            displayText = "Garage Spaces";
        }
        if(displayText!=null) {
            displayText = getString(R.string.invalid) + displayText;
        }
        return displayText;
    }

    private static int extractInt(EditText text) {
        int result = -1;
        try {
            String value = text.getText().toString();
            result = Integer.parseInt(value);
        } catch (NumberFormatException e) {
            // ignore since returning a negative value to signify an invalid extraction
        }
        return result;
    }

    private static double extractDouble(EditText text) {
        double result = -1.0;
        try {
            String value = text.getText().toString();
            result = Double.parseDouble(value);
        } catch (NumberFormatException e) {
            // ignore since returning a negative value to signify an invalid extraction
        }
        return result;
    }

    private TextWatcher textWatcher = new TextWatcher() {
        @Override
        public void beforeTextChanged(CharSequence charSequence, int i, int j, int k) {
        }

        @Override
        public void onTextChanged(CharSequence charSequence, int i, int j, int k) {
        }

        @Override
        public void afterTextChanged(Editable editable) {
            String[] fields = {squareFeet.getText().toString(),
                    fullBaths.getText().toString(),
                    halfBaths.getText().toString(),
                    bedrooms.getText().toString(),
                    windows.getText().toString(),
                    garageSpaces.getText().toString(),
            };
            boolean allFilled = true;
            for(int i=0; i<fields.length && allFilled; ++i) {
                if(fields[i].equals("")) {
                    allFilled = false;
                }
            }
            calculate.setEnabled(allFilled);

            if(!allFilled) {
                output.setVisibility(View.GONE);
            }
        }
    };
}
